import io
import json
import logging
import os


class LKIConfig(dict):
    path = os.path.expanduser("~/.lki.json")

    def __init__(self, seq=None, **kwargs):
        super(LKIConfig, self).__init__(seq=seq, **kwargs)
        self._load_config()

    def __setitem__(self, key, value):
        super(LKIConfig, self).__setitem__(key, value)
        self._save_config()

    def _load_config(self):
        if not os.path.isfile(self.path):
            return
        try:
            with io.open(self.path, "r", encoding="utf-8") as f:
                self.update(json.loads(f.read()))
        except Exception as ex:
            logging.warning(ex)

    def _save_config(self):
        try:
            with io.open(os.path.expanduser(self.path), "w", encoding="utf-8") as f:
                f.write(json.dumps(self, ensure_ascii=False, indent=2))
        except Exception as ex:
            logging.warning(ex)



import datetime
import os
import pathlib
import re
import shutil
import sys

import click

from lki.utils import check_executable, check_file, is_windows, make_link, run

REGEX_GIT_URLS = (
    re.compile(r"^git@([^:]+):(.*?)(.git)?$"),
    re.compile(r"^https?://([^/]+)/(.*?)(.git)?$"),
    re.compile(r"^ssh://git@([^/]+)/(.*?)(.git)?$"),
)
HOME = pathlib.Path("~").expanduser()


@click.group()
def entry():
    """Lirian Su's useful configuration and shortcuts.
    This command will helps you swim through windows and unix,
    git and vim, docker and k8s, and all the fancy.

    See also: https://github.com/LKI/LKI"""


@entry.command()
def install():
    """install configuration to ~/.lki/

    Examples:
        lki install
    """
    check_executable("git")
    repo_path = HOME.joinpath(".lki")
    if not repo_path.exists():
        run(f"git clone -o o --recursive https://github.com/LKI/LKI.git {repo_path}")
    else:
        run(f"git -C {repo_path} pull --rebase")

    def _link(src, dst=None):
        target = HOME.joinpath(dst or src)
        if os.path.exists(target):
            os.remove(target)
        make_link(repo_path.joinpath(src), target, force=True)

    try:
        _link(".gitconfig")
        _link(".gitignore")
        _link(".ideavimrc")
        _link(".inputrc")
        _link(".profile")
        _link(".tmux.conf")
        if is_windows:
            _link(
                ".windows-terminal.json",
                "AppData/Local/Packages/Microsoft.WindowsTerminal_8wekyb3d8bbwe/LocalState/settings.json",
            )
        # TODO: implement `lki install --vim`
        # _link("dotvim/vimrc", ".vimrc")
        #
        # if IS_WIN32:
        #     _link("dotvim/vimrc", "_vimrc")
        #     _link("dotvim", "vimfiles", target_is_directory=True)
        #     _link("dotvim", ".vim", target_is_directory=True)
    except OSError as ex:
        click.echo(
            f"OSError: {ex}\n"
            "  Please check your permissions.\n"
            "Hint: windows requires admin permission when creating symlink.",
            err=True,
        )
        sys.exit(1)


@entry.command()
@click.argument("target")
@click.argument("link_path")
@click.option("-f", "--force", is_flag=True, default=False)
def link(target, link_path, force):
    """link target to link_path

    Examples:
        lki link /usr/bin/python3 /usr/bin/python
    """
    make_link(target, link_path, force=force)


@entry.command()
@click.argument("path", default=str(HOME.joinpath(".lki", ".pre-commit")))
@click.option("-f", "--force", is_flag=True, default=False)
def hook(path, force):
    """install git hook

    Examples:
        lki hook -f
    """
    make_link(path, ".git/hooks/pre-commit", force=force)


@entry.command()
@click.argument("key", required=False, default=None)
@click.argument("value", required=False, default=None)
def env(key, value):
    """list/grep/set environment variable (windows only)"""
    if not is_windows:
        click.echo("lki env only support windows currently", err=True)
        sys.exit(1)
    mute_keys = ("PS1", "_")
    data = {k.lower(): v for k, v in os.environ.items() if k not in mute_keys}
    if key is None:
        for k, v in data.items():
            click.echo(f"{k}={v}")
    elif value is None:
        if key.lower() in data:
            click.echo(data[key.lower()])
        else:
            for k, v in data.items():
                if key.lower() in k:
                    click.echo(f"{k}={v[:100]}")
    else:
        if not value:
            value = "''"
        click.echo(f"Setting environment {key} to {value}")
        run("SETX {} {}".format(key, value))


@entry.command()
@click.argument("url")
def clone(url):
    """clone a git repository to workspace

    Examples:
        lki clone git@github.com:zaihui/hutils.git
    Equals to:
        git clone -o o git@github.com:zaihui/hutils.git {workspace}/github.com/zaihui/hutils

    """
    check_executable("git")
    if not url.startswith("git@") and not url.startswith("http") and not url.startswith("ssh:"):
        if url.count("/") == 1:
            if HOME.joinpath(".ssh", "id_rsa").exists():
                url = f"git@github.com:{url}.git"
            else:
                url = f"https://github.com/{url}.git"
        else:
            url = "https://{}".format(url)
    match = next((m for m in (e.search(url) for e in REGEX_GIT_URLS) if m), None)
    if not match:
        click.echo(f"lki can not understand git url: {url}", err=True)
        sys.exit(1)
    domain, project, _ = match.groups()  # type: str
    domain = domain.split(":")[0]
    workspace = HOME.joinpath("code", "src")
    path = workspace.joinpath(domain, project)
    run(f"git clone -o o {url} {path}")


@entry.command()
def boost():
    """boost apt speed by changing apt's source

    translate from https://raw.githubusercontent.com/ldsink/toolbox/master/ubuntu-set-aliyun-mirror.sh
    """
    check_executable("apt")
    check_file("/etc/apt/sources.list")
    backup_file = datetime.datetime.now().strftime("/etc/apt/sources.list.%Y%m%d%H%M%S.bak")
    print("Backing up at {}".format(backup_file))
    shutil.copyfile("/etc/apt/sources.list", backup_file)
    run(
        r'sed -i -E "s/deb (ht|f)tp(s?)\:\/\/[0-9a-zA-Z]'
        r"([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)"
        r"([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_]*)?\/ubuntu/deb"
        r' http\:\/\/mirrors\.aliyun\.com\/ubuntu/g" /etc/apt/sources.list'
    )


if __name__ == "__main__":
    entry()



import ctypes
import distutils.spawn as spawn
import os
import sys

import click

from lki import error


def check_executable(name):
    """check if executable exists. (raise exeception if not)"""
    if not spawn.find_executable(name):
        raise error.LKIError(f"there is no {name} executable")


def check_file(path):
    """check if file exists. (raise exeception if not)"""
    if not os.path.exists(path):
        raise error.LKIError(f"there is no {path} file")


def full_path(*paths):
    def _f(p):
        return os.path.abspath(os.path.expanduser(str(p)))

    if len(paths) == 1:
        return _f(paths[0])
    return map(_f, paths)


def is_superuser():
    if is_windows:
        return bool(ctypes.windll.shell32.IsUserAnAdmin())
    return os.getuid() == 0


is_windows = bool(sys.platform == "win32")


def make_link(target, link_path, force=False):
    error.LinkError.check(is_windows and not is_superuser(), "please run as admin to link files")

    target, link_path = full_path(target, link_path)
    if os.path.lexists(link_path):
        if force:
            rm(link_path)
        else:
            raise error.LinkError(f"{link_path} exists, specify --force to override.")

    kwargs = {}
    if is_windows and os.path.isdir(target):
        kwargs["target_is_directory"] = True
    os.symlink(target, link_path, **kwargs)


def rm(path):
    if os.path.isdir(path):
        os.rmdir(path)
    else:
        os.remove(path)


def run(*commands):
    """run commands"""
    for command in commands:
        click.echo(f"(Running)> {command}")
        os.system(command)



import io

import setuptools
from pipenv.project import Project
from pipenv.utils.dependencies import convert_deps_to_pip

import lki

with io.open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

install_requires: list[str] = convert_deps_to_pip(Project(chdir=False).parsed_pipfile["packages"])

setuptools.setup(
    name="lki",
    version=lki.__version__,
    description="connect Lirian's useful commands",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Lirian Su",
    author_email="liriansu@gmail.com",
    url="https://github.com/LKI/LKI",
    license="MIT License",
    install_requires=install_requires,
    packages=setuptools.find_packages(),
    entry_points={"console_scripts": "lki = lki.cmdline:entry"},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: Unix",
    ],
)
